# -*- coding: utf-8 -*-
#
# django-codenerix
#
# Copyright 2017 Centrologic Computational Logistic Center S.L.
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.template import Library
from codenerix.templatetags_special import txt2img, ifusergroup

def fbuilder2(f): return lambda arg1,arg2: f(arg1,arg2)
def ftxt2img(arg1, arg2=14, arg3="#ffffff",arg4="#000000", arg5="FreeMono.ttf"):
    return txt2img(arg1, arg2, arg3, arg4, arg5)


register = Library()
register.filter('txt2img',ftxt2img)
register.tag('ifusergroup',fbuilder2(ifusergroup))
